@@ -11,7 +11,7 @@ use integer ; # Forces us to quote all hash keys in 5.004.
 use Carp ;
 
 use vars qw( $VERSION @ISA @EXPORT_OK %EXPORT_TAGS ) ;
-$VERSION     = '1.10' ;
+$VERSION     = '1.11' ;
 
 use Exporter() ;
 
@@ -136,7 +136,7 @@ sub new_from_string { # Class and object method
 
     if( defined $arg[0] ) {
         # 1, 0 and -1 pass through unchanged; -1 will be silently converted to
-        # 0 except for 3-value logic in $class->new
+        # 0 except for 3-degree logic in $class->new
         $arg[0] = TRUE()  if $arg[0] =~ /^-?[tT]/o ;
         $arg[0] = FALSE() if $arg[0] =~ /^-?[fF]/o ;
         $arg[0] = UNDEF() if $arg[0] =~ /^-?[uU]/o ; 
@@ -187,18 +187,18 @@ sub new { # Class and object method
 
     $self->{KEY_VALUE()} = DEF_VALUE() if $self->{KEY_VALUE()} !~ /^(?:\d+|-1)$/o ;
 
-    if( $self->{KEY_DEGREE()} == 2 ) {      # 2-value logic
+    if( $self->{KEY_DEGREE()} == 2 ) {      # 2-degree logic
         $self->{KEY_VALUE()} = ( $self->{KEY_VALUE()} CORE::and 
                                  $self->{KEY_VALUE()} != UNDEF() ) ? 
                                     TRUE() : FALSE() ;
         delete $self->{KEY_PROPAGATE()} ;   # Don't store what we don't use
     }
-    elsif( $self->{KEY_DEGREE()} == 3 ) {   # 3-value logic
+    elsif( $self->{KEY_DEGREE()} == 3 ) {   # 3-degree logic
         if( $self->{KEY_VALUE()} != UNDEF() ) {
             $self->{KEY_VALUE()} = $self->{KEY_VALUE()} ? TRUE() : FALSE() ;
         }
     }
-    else {                                  # Multi-value logic
+    else {                                  # Multi-degree logic
         $self->{KEY_VALUE()} = FALSE() if $self->{KEY_VALUE()} == UNDEF() ;
         $self->{KEY_VALUE()} = $self->{KEY_DEGREE()} 
         if $self->{KEY_VALUE()} > $self->{KEY_DEGREE()} ;
@@ -219,7 +219,7 @@ use overload
         '^'        => \&xor,
         '!'        => \&not,
         # Avoid surprises
-        '='        => sub { croak "=() unsupported" },
+        '='        => sub { croak "=() not overloaded" },
         '+'        => sub { croak "+() unsupported" },
         '-'        => sub { croak "-() unsupported" },
         '*'        => sub { croak "*() unsupported" },
@@ -241,12 +241,12 @@ use overload
         'le'       => sub { croak "le() unsupported" },
         'gt'       => sub { croak "gt() unsupported" },
         'ge'       => sub { croak "ge() unsupported" },
-        'eq'       => sub { croak "eq() unsupported" },
-        'ne'       => sub { croak "ne() unsupported" },
+        'eq'       => sub { croak "eq() unsupported; use == instead" },
+        'ne'       => sub { croak "ne() unsupported; use != instead" },
         '**='      => sub { croak "**=() unsupported" },
         '<<='      => sub { croak "<<=() unsupported" },
         '>>='      => sub { croak ">>=() unsupported" },
-        'cmp'      => sub { croak "cmp() unsupported" },
+        'cmp'      => sub { croak "cmp() unsupported; use <=> instead" },
         'neg'      => sub { croak "neg() unsupported" },
         'nomethod' => sub { croak @_ . "() unsupported" },
         ;
@@ -262,14 +262,14 @@ sub value { # Object method
     if( defined $value ) {
         my $result ;
 
-        if( $self->degree == 2 ) {      # 2-value logic
+        if( $self->degree == 2 ) {      # 2-degree logic
             $result = ( $value CORE::and $value != UNDEF() ) ? TRUE() : FALSE() ;
         }
-        elsif( $self->degree == 3 ) {   # 3-value logic
+        elsif( $self->degree == 3 ) {   # 3-degree logic
             $result = $value ? TRUE() : FALSE() ;
             $result = UNDEF() if $value == UNDEF() ;
         }
-        else {                          # Multi-value logic
+        else {                          # Multi-degree logic
             $result = $value ;
             # UNDEF() is -1 which doesn't match the pattern, hence we can
             # abbreviate the following line
@@ -367,14 +367,14 @@ sub as_string { # Object method
 
     my $result = '' ;
 
-    if( $self->degree == 2 ) {      # 2-value logic
+    if( $self->degree == 2 ) {      # 2-degree logic
         $result = $self->value ? STR_TRUE() : STR_FALSE() ;
     }
-    elsif( $self->degree == 3 ) {   # 3-value logic
+    elsif( $self->degree == 3 ) {   # 3-degree logic
         $result = $self->value ? STR_TRUE() : STR_FALSE() ;
         $result = STR_UNDEF() if $self->value == UNDEF() ;
     }
-    else {                          # Multi-value logic
+    else {                          # Multi-degree logic
         if( $self->value == FALSE() ) {
             $result = STR_FALSE() ;
         }
@@ -410,10 +410,10 @@ sub and { # Object method
     my $value ;
     my $result = $self->new ;
 
-    if( $self->degree == 2 ) {      # 2-value logic
+    if( $self->degree == 2 ) {      # 2-degree logic
         $value = ( $self->value CORE::and $comp->value ) ? TRUE() : FALSE() ;
     }
-    elsif( $self->degree == 3 ) {   # 3-value logic
+    elsif( $self->degree == 3 ) {   # 3-degree logic
         if( $self->propagate ) {
             if( $self->value == UNDEF() CORE::or $comp->value == UNDEF() ) {
                 # At least one is undefined which propagates.
@@ -443,7 +443,7 @@ sub and { # Object method
             }
         }
     }
-    else {                          # Multi-value logic
+    else {                          # Multi-degree logic
         # and is the lowest value
         $value = $self->value < $comp->value ? $self->value : $comp->value ;
     }
@@ -469,10 +469,10 @@ sub or { # Object method
     my $value ;
     my $result = $self->new ;
 
-    if( $self->degree == 2 ) {      # 2-value logic
+    if( $self->degree == 2 ) {      # 2-degree logic
         $value = ( $self->value CORE::or $comp->value ) ? TRUE() : FALSE() ;
     }
-    elsif( $self->degree == 3 ) {   # 3-value logic
+    elsif( $self->degree == 3 ) {   # 3-degree logic
         if( $self->propagate ) {
             if( $self->value == UNDEF() CORE::or $comp->value == UNDEF() ) {
                 # At least one is undefined which propagates.
@@ -502,7 +502,7 @@ sub or { # Object method
             }
         }
     }
-    else {                          # Multi-value logic
+    else {                          # Multi-degree logic
         # or is the greatest value
         $value = $self->value > $comp->value ? $self->value : $comp->value ;
     }
@@ -528,10 +528,10 @@ sub xor { # Object method
     my $value ;
     my $result = $self->new ;
 
-    if( $self->degree == 2 ) {      # 2-value logic
+    if( $self->degree == 2 ) {      # 2-degree logic
         $value = ( $self->value CORE::xor $comp->value ) ? TRUE() : FALSE() ;
     }
-    elsif( $self->degree == 3 ) {   # 3-value logic
+    elsif( $self->degree == 3 ) {   # 3-degree logic
         # Same truth table whether propagating or not.
         if( $self->value == UNDEF() CORE::or $comp->value == UNDEF() ) {
             # At least one is undefined which propagates.
@@ -546,7 +546,7 @@ sub xor { # Object method
             $value = TRUE() ;
         }
     }
-    else {                          # Multi-value logic
+    else {                          # Multi-degree logic
         # By truth table xor(a,b) == and(or(a,b),not(and(a,b)))
         # We could write it thus, but prefer not to use overloading within the
         # module itself:
@@ -570,10 +570,10 @@ sub not { # Object method
     my $value ;
     my $result = $self->new ;
 
-    if( $self->degree == 2 ) {      # 2-value logic
+    if( $self->degree == 2 ) {      # 2-degree logic
         $value = ( $self->value ? FALSE() : TRUE() ) ;
     }
-    elsif( $self->degree == 3 ) {   # 3-value logic
+    elsif( $self->degree == 3 ) {   # 3-degree logic
         # Same truth table whether propagating or not.
         if( $self->value == UNDEF() ) {
             # It's undefined which propogates.
@@ -588,7 +588,7 @@ sub not { # Object method
             $value = TRUE() ;
         }
     }
-    else {                          # Multi-value logic
+    else {                          # Multi-degree logic
         $value = $self->degree - $self->value ;
     }
 
@@ -623,14 +623,14 @@ Math::Logic - Provides pure 2, 3 or multi-value logic.
 
     use Math::Logic ':STR' ; # STR_TRUE STR_FALSE STR_UNDEF
 
-    # 2-value logic
+    # 2-degree logic
     my $true  = Math::Logic->new( -value => TRUE,  -degree => 2 ) ;
     my $false = Math::Logic->new( -value => FALSE, -degree => 2 ) ;
     my $x     = Math::Logic->new_from_string( 'TRUE,2' ) ;
 
     print "true" if $true ;
 
-    # 3-value logic (non-propagating)
+    # 3-degree logic (non-propagating)
     my $true  = Math::Logic->new( -value => TRUE,  -degree => 3 ) ;
     my $false = Math::Logic->new( -value => FALSE, -degree => 3 ) ;
     my $undef = Math::Logic->new( -value => UNDEF, -degree => 3 ) ;
@@ -638,7 +638,7 @@ Math::Logic - Provides pure 2, 3 or multi-value logic.
 
     print "true" if ( $true | $undef ) == TRUE ;
 
-    # 3-value logic (propagating)
+    # 3-degree logic (propagating)
     my $true  = Math::Logic->new( -value => TRUE,  -degree => 3, -propagate => 1 ) ;
     my $false = Math::Logic->new( -value => FALSE, -degree => 3, -propagate => 1 ) ;
     my $undef = Math::Logic->new( -value => UNDEF, -degree => 3, -propagate => 1 ) ;
@@ -646,7 +646,7 @@ Math::Logic - Provides pure 2, 3 or multi-value logic.
 
     print "undef" if ( $true | $undef ) == UNDEF ;
 
-    # multi-value logic
+    # multi-degree logic
     my $TRUE   = 100 ; # Define our own true
     my $FALSE  = FALSE ;
     my $true   = Math::Logic->new( -value => $TRUE,  -degree => $TRUE ) ;
@@ -674,23 +674,27 @@ true or false. In fact perl sometimes returns 0 and sometimes returns undef
 for false depending on the operator and the order of the arguments. For "true"
 Perl generally returns the first value that evaluated to true which turns out
 to be extremely useful in practice. Given the choice Perl's built-in logical
-operators are to be preferred -- but when you really want pure 2-value logic
-or 3-value logic or multi-value logic they are available through this module.
+operators are to be preferred -- but when you really want pure 2-degree logic
+or 3-degree logic or multi-degree logic they are available through this module.
 
-The only 2-value logic values are 1 (TRUE) and 0 (FALSE).
+The only 2-degree logic values are 1 (TRUE) and 0 (FALSE).
 
-The only 3-value logic values are 1 (TRUE), 0 (FALSE) and -1 (UNDEF). Note
+The only 3-degree logic values are 1 (TRUE), 0 (FALSE) and -1 (UNDEF). Note
 that UNDEF is -1 I<not> C<undef>!
 
-The only multi-value logic values are 0 (FALSE)..C<-degree> -- the value of
+The only multi-degree logic values are 0 (FALSE)..C<-degree> -- the value of
 TRUE is equal to the degree, usually 100.
 
+The C<-degree> is the maximum value (except for 2 and 3-degree logic); i.e.
+logic of I<n>-degree is I<n+1>-value logic, e.g. 100-degree logic has 101
+values, 0..100.
+
 Although some useful constants may be exported, this is an object module and
 the results of logical comparisons are Math::Logic objects.
 
-=head2 2-value logic
+=head2 2-degree logic
 
-2-value logic has one simple truth table for each logical operator.
+2-degree logic has one simple truth table for each logical operator.
 
         Perl Logic      Perl Logic     Perl Logic 
     A B and  and    A B or   or    A B xor  xor
@@ -719,9 +723,9 @@ value is either 0 (FALSE) or 1 (TRUE) only.
 
     print $result if $result == FALSE ; 
 
-=head2 3-value logic
+=head2 3-degree logic
 
-3-value logic has two different truth tables for "and" and "or"; this module
+3-degree logic has two different truth tables for "and" and "or"; this module
 supports both. In the Perl column F means false or undefined; and T, F and U
 under Math::Logic are objects with values 1 (TRUE), 0 (FALSE) and -1 (UNDEF)
 respectively. The + signifies propagating nulls (UNDEFs).
@@ -747,7 +751,7 @@ respectively. The + signifies propagating nulls (UNDEFs).
     F  T    T    T
     T  F    F    F
 
-    # 3-value logic (non-propagating)
+    # 3-degree logic (non-propagating)
     my $true   = Math::Logic->new( -value => TRUE,  -degree => 3 ) ;
     my $false  = Math::Logic->new( -value => FALSE, -degree => 3 ) ;
     my $undef  = Math::Logic->new( -value => UNDEF, -degree => 3 ) ;
@@ -756,7 +760,7 @@ respectively. The + signifies propagating nulls (UNDEFs).
 
     print $result if $result == FALSE ; 
 
-    # 3-value logic (propagating)
+    # 3-degree logic (propagating)
     my $true   = Math::Logic->new( -value => TRUE,  -degree => 3, -propagate => 1 ) ;
     my $false  = Math::Logic->new( -value => FALSE, -degree => 3, -propagate => 1 ) ;
     my $undef  = Math::Logic->new( -value => UNDEF, -degree => 3, -propagate => 1 ) ;
@@ -765,13 +769,13 @@ respectively. The + signifies propagating nulls (UNDEFs).
 
     print $result if $result == UNDEF ; 
 
-=head2 multi-value logic
+=head2 multi-degree logic
 
 This is used in `fuzzy' logic. Typically we set the C<-degree> to 100
 representing 100% likely, i.e. true; 0 represents 0% likely, i.e. false, and
 any integer in-between is a probability.
 
-The truth tables for multi-value logic work like this:
+The truth tables for multi-degree logic work like this:
 
     and     lowest  value is the result;
     or      highest value is the result;
@@ -805,7 +809,7 @@ The truth tables for multi-value logic work like this:
      67  33
     100   0
    
-    # multi-value logic
+    # multi-degree logic
     my $TRUE   = 100 ; # Define our own TRUE and FALSE
     my $FALSE  = FALSE ;
     $true      = Math::Logic->new( -value => $TRUE,  -degree => $TRUE ) ;
@@ -851,7 +855,7 @@ The truth tables for multi-value logic work like this:
 
 This creates new Math::Logic objects. C<new> should never fail because it will
 munge any arguments into something `sensible'; in particular if the value is
-set to -1 (UNDEF) for 2 or multi-value logic it is silently converted to 0
+set to -1 (UNDEF) for 2 or multi-degree logic it is silently converted to 0
 (FALSE). In all other cases anything that is true in Perl is converted to 1
 (TRUE) and everything else to 0 (FALSE).
 
@@ -864,12 +868,12 @@ C<-degree> an integer indicating the number of possible truth values;
 typically set to 2, 3 or 100 (to represent percentages). Minimum value is 2.
 
 C<-propagate> a true/false integer indicating whether NULLs (UNDEF) should
-propagate; only applicable for 3-value logic where it influences which truth
+propagate; only applicable for 3-degree logic where it influences which truth
 table is used. 
 
-C<-value> an integer representing the truth value. For 2-value logic only 1
-and 0 are valid (TRUE and FALSE); for 3-value logic 1, 0, and -1 are valid
-(TRUE, FALSE and UNDEF); for multi-value logic any positive integer less than
+C<-value> an integer representing the truth value. For 2-degree logic only 1
+and 0 are valid (TRUE and FALSE); for 3-degree logic 1, 0, and -1 are valid
+(TRUE, FALSE and UNDEF); for multi-degree logic any positive integer less than
 or equal to the C<-degree> is valid.
 
 =head2 new_from_string (class and object method)
@@ -898,9 +902,9 @@ parameter or include it as 0 (zero).
     print $x->value ;
     print $x ;
 
-This returns the numeric value of the object. For 2-value logic this will
-always be 1 or 0; for 3-value logic the value will be 1, 0 or -1; for
-multi-value logic the value will be a positive integer <= C<-degree>. 
+This returns the numeric value of the object. For 2-degree logic this will
+always be 1 or 0; for 3-degree logic the value will be 1, 0 or -1; for
+multi-degree logic the value will be a positive integer <= C<-degree>. 
 
 =head2 degree (object method)
 
@@ -914,7 +918,7 @@ values the object may hold; it is always 2 or more.
     print $x->propagate ;
 
 This returns whether or not the object propagates NULLs (UNDEF). Objects using
-2 or multi-value logic always return FALSE; 3-value logic objects may return
+2 or multi-degree logic always return FALSE; 3-degree logic objects may return
 TRUE or FALSE.
 
 =head2 incompatible (object method)
@@ -926,15 +930,15 @@ incompatible (which Perl treats as TRUE), e.g.:
 
     $x = Math::Logic->new_from_string('1,2') ;
     $y = Math::Logic->new_from_string('0,3') ;
-    # The above are incompatible because the first uses 2-value logic and the
-    # second uses 3-value logic.
+    # The above are incompatible because the first uses 2-degree logic and the
+    # second uses 3-degree logic.
     print $x->incompatible( $y ) if $x->incompatible( $y ) ;
     # This will print something like:
     Math::Logic(2,0) and Math::Logic(3,0) are incompatible at ./logic.t line 2102
     # The first number given is the degree and the second the propagate setting
 
 Objects are compatible if they have the same C<-degree> and in the case of
-3-value logic the same C<-propagate>. Logical operators will only work on
+3-degree logic the same C<-propagate>. Logical operators will only work on
 compatible objects, there is no type-coersion (but see typecasting later).
 
 =head2 compatible DEPRECATED (object method)
@@ -943,7 +947,7 @@ compatible objects, there is no type-coersion (but see typecasting later).
 
 Returns TRUE or FALSE depending on whether the two objects are compatible.
 Objects are compatible if they have the same C<-degree> and in the case of
-3-value logic the same C<-propagate>. Logical operators will only work on
+3-degree logic the same C<-propagate>. Logical operators will only work on
 compatible objects, there is no type-coersion (but see typecasting later).
 
 =head2 as_string and "" (object method)
@@ -975,7 +979,7 @@ C<as_string>.
     print "true" if $y->and( $z ) == TRUE ;
 
 Applies logical and to two objects. The truth table used depends on the
-object's C<-degree> (and in the case of 3-value logic on the C<-propagate>).
+object's C<-degree> (and in the case of 3-degree logic on the C<-propagate>).
 (See the truth tables above.)
 
 =head2 or and | (object method)
@@ -989,7 +993,7 @@ object's C<-degree> (and in the case of 3-value logic on the C<-propagate>).
     print "true" if $y->or( $z ) == TRUE ;
 
 Applies logical or to two objects. The truth table used depends on the
-object's C<-degree> (and in the case of 3-value logic on the C<-propagate>).
+object's C<-degree> (and in the case of 3-degree logic on the C<-propagate>).
 (See the truth tables above.)
 
 =head2 xor and ^ (object method)
@@ -1023,12 +1027,12 @@ objects, i.e. <, <=, >, =>, ==, != and <=>.
 
 =head2 typecasting
 
-The only typecasting that appears to make sense is between 2 and 3-value
+The only typecasting that appears to make sense is between 2 and 3-degree
 logic. There is no direct support for it but it can be achieved thus:
 
-    my $x = Math::Logic->new_from_string( '1,2' ) ;  # TRUE  2-value
-    my $y = Math::Logic->new_from_string( '0,3' ) ;  # FALSE 3-value
-    my $z = Math::Logic->new_from_string( '-1,3' ) ; # UNDEF 3-value
+    my $x = Math::Logic->new_from_string( '1,2' ) ;  # TRUE  2-degree
+    my $y = Math::Logic->new_from_string( '0,3' ) ;  # FALSE 3-degree
+    my $z = Math::Logic->new_from_string( '-1,3' ) ; # UNDEF 3-degree
 
     $x3 = $x->new( -degree => 3 ) ;
     $y2 = $y->new( -degree => 2 ) ;
@@ -1036,13 +1040,25 @@ logic. There is no direct support for it but it can be achieved thus:
 
 =head1 BUGS
 
-(none known)
+Multi-degree logic has a minimum degree of 4, i.e. 5-value, 0..4.  
+
+If you use & on two incompatible Math::Logic objects perl dies; I believe that
+this is due to a problem with overload.
 
 =head1 CHANGES
 
+2000/02/27
+
+Numerous minor documentation changes to clarify terminology.
+
+Two bugs noted.
+
+More tests added.
+
+
 2000/02/23
 
-Corrected multi-value xor to match the truth table equivalence, i.e. 
+Corrected multi-degree xor to match the truth table equivalence, i.e. 
 
     xor(a,b) == and(or(a,b),not(and(a,b)))
 
